@@ -65,7 +65,7 @@
 
 #define VERSION_MAJOR 0
 #define VERSION_MINOR 1
-#define VERSION "0.3"
+#define VERSION "0.4"
 
 #ifdef WIN32
 #undef WITH_THREAD /* Thread support locks up xchat on Win32. */
@@ -1671,7 +1671,7 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 	xchat_list *list;
 	PyObject *l;
 	const char *name;
-	const char **fields;
+	const char *const *fields;
 	int i;
 
 	if (!PyArg_ParseTuple(args, "s:get_list", &name))
@@ -1726,6 +1726,7 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 						(xchat_context*)sattr);
 					break;
 				}
+			default: /* ignore unknown (newly added?) types */
 				continue;
 			}
 			if (attr == NULL)
@@ -1750,7 +1751,7 @@ static PyObject *
 Module_xchat_get_lists(PyObject *self, PyObject *args)
 {
 	PyObject *l, *o;
-	const char **fields;
+	const char *const *fields;
 	int i;
 	/* This function is thread safe, and returns statically
 	 * allocated data. */
