@@ -273,6 +273,29 @@ cmd_allchannels (session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_allchannelslocal (session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	GSList *list = sess_list;
+	server *serv = sess->server;
+
+	if (!*word_eol[2])
+		return FALSE;
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->type == SESS_CHANNEL && sess->channel[0] &&
+			 sess->server->connected && sess->server == serv)
+		{
+			handle_command (sess, word_eol[2], FALSE);
+		}
+		list = list->next;
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_allservers (struct session *sess, char *tbuf, char *word[],
 					 char *word_eol[])
@@ -511,8 +534,12 @@ cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	server *serv = sess->server;
 	const char *locale = NULL;
+	int offset = 0;
 
-	if (!word[2][0])
+	if (strcmp (word[2], "-quiet") == 0)
+		offset++;
+
+	if (!word[2 + offset][0])
 	{
 		g_get_charset (&locale);
 		PrintTextf (sess, "Current charset: %s\n",
@@ -520,15 +547,14 @@ cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		return TRUE;
 	}
 
-	if (servlist_check_encoding (word[2]))
+	if (servlist_check_encoding (word[2 + offset]))
 	{
-		if (serv->encoding)
-			free (serv->encoding);
-		serv->encoding = strdup (word[2]);
-		PrintTextf (sess, "Charset changed to: %s\n", word[2]);
+		server_set_encoding (serv, word[2 + offset]);
+		if (offset < 1)
+			PrintTextf (sess, "Charset changed to: %s\n", word[2 + offset]);
 	} else
 	{
-		PrintTextf (sess, "\0034Unknown charset:\017 %s\n", word[2]);
+		PrintTextf (sess, "\0034Unknown charset:\017 %s\n", word[2 + offset]);
 	}
 
 	return TRUE;
@@ -621,6 +647,20 @@ cmd_country (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	char *code = word[2];
 	if (*code)
 	{
+		/* search? */
+		if (strcmp (code, "-s") == 0)
+		{
+			country_search (word[3], sess, (void *)PrintTextf);
+			return TRUE;
+		}
+
+		/* search, but forgot the -s */
+		if (strchr (code, '*'))
+		{
+			country_search (code, sess, (void *)PrintTextf);
+			return TRUE;
+		}
+
 		sprintf (tbuf, "%s = %s\n", code, country (code));
 		PrintText (sess, tbuf);
 		return TRUE;
@@ -1323,6 +1363,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int fds[2], pid = 0;
 	struct nbexec *s;
 	int shell = TRUE;
+	int fd;
 
 	if (*cmd)
 	{
@@ -1392,10 +1433,12 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			dup2 (fds[1], 2);
 			/* Also copy it to stdin so we can write to it */
 			dup2 (fds[1], 0);
+			/* Now close all open file descriptors except stdin, stdout and stderr */
+			for (fd = 3; fd < 1024; fd++) close(fd);
 			/* Now we call /bin/sh to run our cmd ; made it more friendly -DC1 */
 			if (shell)
 			{
-				execl ("/bin/sh", "sh", "-c", cmd, 0);
+				execl ("/bin/sh", "sh", "-c", cmd, NULL);
 			} else
 			{
 				char **argv;
@@ -1565,6 +1608,12 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	} else if (!strcasecmp (word[2], "ICONIFY"))
 	{
 		fe_ctrl_gui (sess, 5, 0);
+	} else if (!strcasecmp (word[2], "MENU"))
+	{
+		if (!strcasecmp (word[3], "TOGGLE"))
+			fe_ctrl_gui (sess, 6, 0);
+		else
+			return FALSE;
 	} else
 	{
 		return FALSE;
@@ -2281,6 +2330,22 @@ cmd_say (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_setcursor (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int delta = FALSE;
+
+	if (*word[2])
+	{
+		if (word[2][0] == '-' || word[2][0] == '+')
+			delta = TRUE;
+		fe_set_inputbox_cursor (sess, delta, atoi (word[2]));
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_settab (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2295,6 +2360,18 @@ cmd_settab (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_settext (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (*word_eol[2])
+	{
+		fe_set_inputbox_contents (sess, word_eol[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2484,6 +2561,18 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_url (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[2][0])
+	{
+		fe_open_url (word[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 userlist_cb (struct User *user, session *sess)
 {
@@ -2620,26 +2709,28 @@ cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 /* *MUST* be kept perfectly sorted for the bsearch to work */
 const struct commands xc_cmds[] = {
-	{"ADDBUTTON", cmd_addbutton, 0, 0,
+	{"ADDBUTTON", cmd_addbutton, 0, 0, 1,
 	 N_("ADDBUTTON <name> <action>, adds a button under the user-list")},
-	{"ALLCHAN", cmd_allchannels, 0, 0,
+	{"ALLCHAN", cmd_allchannels, 0, 0, 1,
 	 N_("ALLCHAN <cmd>, sends a command to all channels you're in")},
-	{"ALLSERV", cmd_allservers, 0, 0,
+	{"ALLCHANL", cmd_allchannelslocal, 0, 0, 1,
+	 N_("ALLCHANL <cmd>, sends a command to all channels you're in")},
+	{"ALLSERV", cmd_allservers, 0, 0, 1,
 	 N_("ALLSERV <cmd>, sends a command to all servers you're in")},
-	{"AWAY", cmd_away, 1, 0, N_("AWAY [<reason>], sets you away")},
-	{"BAN", cmd_ban, 1, 1,
+	{"AWAY", cmd_away, 1, 0, 1, N_("AWAY [<reason>], sets you away")},
+	{"BAN", cmd_ban, 1, 1, 1,
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
-	{"CHARSET", cmd_charset, 0, 0, 0},
-	{"CLEAR", cmd_clear, 0, 0, N_("CLEAR, Clears the current text window")},
-	{"CLOSE", cmd_close, 0, 0, N_("CLOSE, Closes the current window/tab")},
+	{"CHARSET", cmd_charset, 0, 0, 1, 0},
+	{"CLEAR", cmd_clear, 0, 0, 1, N_("CLEAR, Clears the current text window")},
+	{"CLOSE", cmd_close, 0, 0, 1, N_("CLOSE, Closes the current window/tab")},
 
-	{"COUNTRY", cmd_country, 0, 0,
-	 N_("COUNTRY <code>, finds a country code, eg: au = australia")},
-	{"CTCP", cmd_ctcp, 1, 0,
+	{"COUNTRY", cmd_country, 0, 0, 1,
+	 N_("COUNTRY [-s] <code|wildcard>, finds a country code, eg: au = australia")},
+	{"CTCP", cmd_ctcp, 1, 0, 1,
 	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
-	{"CYCLE", cmd_cycle, 1, 1,
+	{"CYCLE", cmd_cycle, 1, 1, 1,
 	 N_("CYCLE, parts current channel and immediately rejoins")},
-	{"DCC", cmd_dcc, 0, 0,
+	{"DCC", cmd_dcc, 0, 0, 1,
 	 N_("\n"
 	 "DCC GET <nick>                     - accept an offered file\n"
 	 "DCC SEND [-maxcps=#] <nick> [file] - send a file to someone\n"
@@ -2647,141 +2738,144 @@ const struct commands xc_cmds[] = {
 	 "DCC CHAT <nick>                    - offer DCC CHAT to someone\n"
 	 "DCC CLOSE <type> <nick> <file>         example:\n"
 	 "         /dcc close send johnsmith file.tar.gz")},
-	{"DEBUG", cmd_debug, 0, 0, 0},
+	{"DEBUG", cmd_debug, 0, 0, 1, 0},
 
-	{"DEHOP", cmd_dehop, 1, 1,
+	{"DEHOP", cmd_dehop, 1, 1, 1,
 	 N_("DEHOP <nick>, removes chanhalf-op status from the nick on the current channel (needs chanop)")},
-	{"DELBUTTON", cmd_delbutton, 0, 0,
+	{"DELBUTTON", cmd_delbutton, 0, 0, 1,
 	 N_("DELBUTTON <name>, deletes a button from under the user-list")},
-	{"DEOP", cmd_deop, 1, 1,
+	{"DEOP", cmd_deop, 1, 1, 1,
 	 N_("DEOP <nick>, removes chanop status from the nick on the current channel (needs chanop)")},
-	{"DEVOICE", cmd_devoice, 1, 1,
+	{"DEVOICE", cmd_devoice, 1, 1, 1,
 	 N_("DEVOICE <nick>, removes voice status from the nick on the current channel (needs chanop)")},
-	{"DISCON", cmd_discon, 0, 0, N_("DISCON, Disconnects from server")},
-	{"DNS", cmd_dns, 0, 0, N_("DNS <nick|host|ip>, Finds a users IP number")},
-	{"ECHO", cmd_echo, 0, 0, N_("ECHO <text>, Prints text locally")},
+	{"DISCON", cmd_discon, 0, 0, 1, N_("DISCON, Disconnects from server")},
+	{"DNS", cmd_dns, 0, 0, 1, N_("DNS <nick|host|ip>, Finds a users IP number")},
+	{"ECHO", cmd_echo, 0, 0, 1, N_("ECHO <text>, Prints text locally")},
 #ifndef WIN32
-	{"EXEC", cmd_exec, 0, 0,
+	{"EXEC", cmd_exec, 0, 0, 1,
 	 N_("EXEC [-o] <command>, runs the command. If -o flag is used then output is sent to current channel, else is printed to current text box")},
 #ifndef __EMX__
-	{"EXECCONT", cmd_execc, 0, 0, N_("EXECCONT, sends the process SIGCONT")},
+	{"EXECCONT", cmd_execc, 0, 0, 1, N_("EXECCONT, sends the process SIGCONT")},
 #endif
-	{"EXECKILL", cmd_execk, 0, 0,
+	{"EXECKILL", cmd_execk, 0, 0, 1,
 	 N_("EXECKILL [-9], kills a running exec in the current session. If -9 is given the process is SIGKILL'ed")},
 #ifndef __EMX__
-	{"EXECSTOP", cmd_execs, 0, 0, N_("EXECSTOP, sends the process SIGSTOP")},
-	{"EXECWRITE", cmd_execw, 0, 0, N_("EXECWRITE, sends data to the processes stdin")},
+	{"EXECSTOP", cmd_execs, 0, 0, 1, N_("EXECSTOP, sends the process SIGSTOP")},
+	{"EXECWRITE", cmd_execw, 0, 0, 1, N_("EXECWRITE, sends data to the processes stdin")},
 #endif
 #endif
-	{"FLUSHQ", cmd_flushq, 0, 0,
+	{"FLUSHQ", cmd_flushq, 0, 0, 1,
 	 N_("FLUSHQ, flushes the current server's send queue")},
-	{"GATE", cmd_gate, 0, 0,
+	{"GATE", cmd_gate, 0, 0, 1,
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
-	{"GETINT", cmd_getint, 0, 0, "GETINT <default> <command> <prompt>"},
-	{"GETSTR", cmd_getstr, 0, 0, "GETSTR <default> <command> <prompt>"},
-	{"GUI", cmd_gui, 0, 0, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]"},
-	{"HELP", cmd_help, 0, 0, 0},
-	{"HOP", cmd_hop, 1, 1,
+	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
+	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
+	{"GUI", cmd_gui, 0, 0, 1, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>]"},
+	{"HELP", cmd_help, 0, 0, 1, 0},
+	{"HOP", cmd_hop, 1, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
-	{"IGNORE", cmd_ignore, 0, 0,
+	{"IGNORE", cmd_ignore, 0, 0, 1,
 	 N_("IGNORE <mask> <types..> <options..>\n"
 	 "    mask - host mask to ignore, eg: *!*@*.aol.com\n"
 	 "    types - types of data to ignore, one or all of:\n"
 	 "            PRIV, CHAN, NOTI, CTCP, DCC, INVI, ALL\n"
 	 "    options - NOSAVE, QUIET")},
 
-	{"INVITE", cmd_invite, 1, 0,
+	{"INVITE", cmd_invite, 1, 0, 1,
 	 N_("INVITE <nick> [<channel>], invites someone to a channel, by default the current channel (needs chanop)")},
-	{"JOIN", cmd_join, 1, 0, N_("JOIN <channel>, joins the channel")},
-	{"KICK", cmd_kick, 1, 1,
+	{"JOIN", cmd_join, 1, 0, 0, N_("JOIN <channel>, joins the channel")},
+	{"KICK", cmd_kick, 1, 1, 1,
 	 N_("KICK <nick>, kicks the nick from the current channel (needs chanop)")},
-	{"KICKBAN", cmd_kickban, 1, 1,
+	{"KICKBAN", cmd_kickban, 1, 1, 1,
 	 N_("KICKBAN <nick>, bans then kicks the nick from the current channel (needs chanop)")},
-	{"KILLALL", cmd_killall, 0, 0, "KILLALL, immediately exit"},
-	{"LAGCHECK", cmd_lagcheck, 0, 0,
+	{"KILLALL", cmd_killall, 0, 0, 1, "KILLALL, immediately exit"},
+	{"LAGCHECK", cmd_lagcheck, 0, 0, 1,
 	 N_("LAGCHECK, forces a new lag check")},
-	{"LASTLOG", cmd_lastlog, 0, 0,
+	{"LASTLOG", cmd_lastlog, 0, 0, 1,
 	 N_("LASTLOG <string>, searches for a string in the buffer")},
-	{"LIST", cmd_list, 1, 0, 0},
-	{"LOAD", cmd_load, 0, 0, N_("LOAD <file>, loads a plugin or script")},
+	{"LIST", cmd_list, 1, 0, 1, 0},
+	{"LOAD", cmd_load, 0, 0, 1, N_("LOAD <file>, loads a plugin or script")},
 
-	{"MDEHOP", cmd_mdehop, 1, 1,
+	{"MDEHOP", cmd_mdehop, 1, 1, 1,
 	 N_("MDEHOP, Mass deop's all chanhalf-ops in the current channel (needs chanop)")},
-	{"MDEOP", cmd_mdeop, 1, 1,
+	{"MDEOP", cmd_mdeop, 1, 1, 1,
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
-	{"ME", cmd_me, 0, 0,
+	{"ME", cmd_me, 0, 0, 1,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
-	{"MKICK", cmd_mkick, 1, 1,
+	{"MKICK", cmd_mkick, 1, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
-	{"MODE", cmd_mode, 1, 0, 0},
-	{"MOP", cmd_mop, 1, 1,
+	{"MODE", cmd_mode, 1, 0, 1, 0},
+	{"MOP", cmd_mop, 1, 1, 1,
 	 N_("MOP, Mass op's all users in the current channel (needs chanop)")},
-	{"MSG", cmd_msg, 0, 0, N_("MSG <nick> <message>, sends a private message")},
+	{"MSG", cmd_msg, 0, 0, 1, N_("MSG <nick> <message>, sends a private message")},
 
-	{"NAMES", cmd_names, 1, 0,
+	{"NAMES", cmd_names, 1, 0, 1,
 	 N_("NAMES, Lists the nicks on the current channel")},
-	{"NCTCP", cmd_nctcp, 1, 0,
+	{"NCTCP", cmd_nctcp, 1, 0, 1,
 	 N_("NCTCP <nick> <message>, Sends a CTCP notice")},
-	{"NEWSERVER", cmd_newserver, 0, 0, N_("NEWSERVER <hostname> [<port>]")},
-	{"NICK", cmd_nick, 0, 0, N_("NICK <nickname>, sets your nick")},
+	{"NEWSERVER", cmd_newserver, 0, 0, 1, N_("NEWSERVER <hostname> [<port>]")},
+	{"NICK", cmd_nick, 0, 0, 1, N_("NICK <nickname>, sets your nick")},
 
-	{"NOTICE", cmd_notice, 1, 0,
+	{"NOTICE", cmd_notice, 1, 0, 1,
 	 N_("NOTICE <nick/channel> <message>, sends a notice. Notices are a type of message that should be auto reacted to")},
-	{"NOTIFY", cmd_notify, 0, 0,
+	{"NOTIFY", cmd_notify, 0, 0, 1,
 	 N_("NOTIFY [<nick>], lists your notify list or adds someone to it")},
-	{"OP", cmd_op, 1, 1,
+	{"OP", cmd_op, 1, 1, 1,
 	 N_("OP <nick>, gives chanop status to the nick (needs chanop)")},
-	{"PART", cmd_part, 1, 1,
+	{"PART", cmd_part, 1, 1, 1,
 	 N_("PART [<channel>] [<reason>], leaves the channel, by default the current one")},
-	{"PING", cmd_ping, 1, 0,
+	{"PING", cmd_ping, 1, 0, 1,
 	 N_("PING <nick | channel>, CTCP pings nick or channel")},
-	{"QUERY", cmd_query, 0, 0,
+	{"QUERY", cmd_query, 0, 0, 1,
 	 N_("QUERY <nick>, opens up a new privmsg window to someone")},
-	{"QUIT", cmd_quit, 0, 0,
+	{"QUIT", cmd_quit, 0, 0, 1,
 	 N_("QUIT [<reason>], disconnects from the current server")},
-	{"QUOTE", cmd_quote, 1, 0,
+	{"QUOTE", cmd_quote, 1, 0, 1,
 	 N_("QUOTE <text>, sends the text in raw form to the server")},
 #ifdef USE_OPENSSL
-	{"RECONNECT", cmd_reconnect, 0, 0,
+	{"RECONNECT", cmd_reconnect, 0, 0, 1,
 	 N_("RECONNECT [-ssl] [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")}, 
 #else 
-	{"RECONNECT", cmd_reconnect, 0, 0,
+	{"RECONNECT", cmd_reconnect, 0, 0, 1,
 	 N_("RECONNECT [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")},
 #endif
-	{"RECV", cmd_recv, 1, 0, N_("RECV <text>, send raw data to xchat, as if it was received from the irc server")},
+	{"RECV", cmd_recv, 1, 0, 1, N_("RECV <text>, send raw data to xchat, as if it was received from the irc server")},
 
-	{"SAY", cmd_say, 0, 0,
+	{"SAY", cmd_say, 0, 0, 1,
 	 N_("SAY <text>, sends the text to the object in the current window")},
 #ifdef USE_OPENSSL
-	{"SERVCHAN", cmd_servchan, 0, 0,
+	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN [-ssl] <host> <port> <channel>, connects and joins a channel")},
 #else
-	{"SERVCHAN", cmd_servchan, 0, 0,
+	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN <host> <port> <channel>, connects and joins a channel")},
 #endif
 #ifdef USE_OPENSSL
-	{"SERVER", cmd_server, 0, 0,
+	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER [-ssl] <host> [<port>] [<password>], connects to a server, the default port is 6667 for normal connections, and 9999 for ssl connections")},
 #else
-	{"SERVER", cmd_server, 0, 0,
+	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
-	{"SET", cmd_set, 0, 0, N_("SET [-quiet] <variable> [<value>]")},
-	{"SETTAB", cmd_settab, 0, 0, 0},
-	{"TOPIC", cmd_topic, 1, 1,
+	{"SET", cmd_set, 0, 0, 1, N_("SET [-quiet] <variable> [<value>]")},
+	{"SETCURSOR", cmd_setcursor, 0, 0, 1, N_("SETCURSOR [-|+]<position>")},
+	{"SETTAB", cmd_settab, 0, 0, 1, 0},
+	{"SETTEXT", cmd_settext, 0, 0, 1, 0},
+	{"TOPIC", cmd_topic, 1, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
-	{"UNBAN", cmd_unban, 1, 1,
+	{"UNBAN", cmd_unban, 1, 1, 1,
 	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
-	{"UNIGNORE", cmd_unignore, 0, 0, N_("UNIGNORE <mask> [QUIET]")},
-	{"UNLOAD", cmd_unload, 0, 0, N_("UNLOAD <name>, unloads a plugin or script")},
-	{"USERLIST", cmd_userlist, 1, 1, 0},
-	{"VOICE", cmd_voice, 1, 1,
+	{"UNIGNORE", cmd_unignore, 0, 0, 1, N_("UNIGNORE <mask> [QUIET]")},
+	{"UNLOAD", cmd_unload, 0, 0, 1, N_("UNLOAD <name>, unloads a plugin or script")},
+	{"URL", cmd_url, 0, 0, 1, N_("URL <url>, opens a URL in your browser")},
+	{"USERLIST", cmd_userlist, 1, 1, 1, 0},
+	{"VOICE", cmd_voice, 1, 1, 1,
 	 N_("VOICE <nick>, gives voice status to someone (needs chanop)")},
-	{"WALLCHAN", cmd_wallchan, 1, 1,
+	{"WALLCHAN", cmd_wallchan, 1, 1, 1,
 	 N_("WALLCHAN <message>, writes the message to all channels")},
-	{"WALLCHOP", cmd_wallchop, 1, 1,
+	{"WALLCHOP", cmd_wallchop, 1, 1, 1,
 	 N_("WALLCHOP <message>, sends the message to all chanops on the current channel")},
-	{0, 0, 0, 0, 0}
+	{0, 0, 0, 0, 0, 0}
 };
 
 
@@ -3331,6 +3425,10 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 	/* split the text into words and word_eol */
 	process_data_init (pdibuf, cmd, word, word_eol, TRUE);
+	int_cmd = find_internal_command (word[1]);
+	/* redo it without quotes processing, for some commands like /JOIN */
+	if (int_cmd && !int_cmd->handle_quotes)
+		process_data_init (pdibuf, cmd, word, word_eol, FALSE);
 
 	if (check_spch && prefs.perc_color)
 		check_special_chars (cmd, prefs.perc_ascii);
