@@ -64,6 +64,25 @@ irc_login (server *serv, char *user, char *realname)
 				  serv->nick, user, hostname, serv->servername, realname);
 }
 
+static void
+irc_nickserv (server *serv, char *pass)
+{
+	switch (serv->nickservtype)
+	{
+	case 0:
+		tcp_sendf (serv, "PRIVMSG NickServ :identify %s\r\n", pass);
+		break;
+	case 1:
+		tcp_sendf (serv, "NICKSERV :identify %s\r\n", pass);
+		break;
+	case 2:
+		tcp_sendf (serv, "NS :identify %s\r\n", pass);
+		break;
+	case 3:
+		tcp_sendf (serv, "PRIVMSG NS :identify %s\r\n", pass);
+	}
+}
+
 static void
 irc_join (server *serv, char *channel, char *key)
 {
@@ -316,6 +335,15 @@ process_numeric (session * sess, int n,
 			if (prefs.ip_from_server)
 				inbound_foundip (sess, strrchr(word[10], '@')+1);
 		}
+
+		/* use /NICKSERV */
+		if (strcasecmp (word[7], "DALnet") == 0)
+			serv->nickservtype = 1;
+
+		/* use /NS */
+		else if (strcasecmp (word[7], "freenode") == 0)
+			serv->nickservtype = 2;
+
 		goto def;
 
 	case 4:	/* check the ircd type */
@@ -349,7 +377,6 @@ process_numeric (session * sess, int n,
 			serv->have_idmsg = TRUE;
 			break;
 		}
-		serv->have_idmsg = FALSE;
 		goto def;
 
 	case 301:
@@ -587,6 +614,20 @@ process_numeric (session * sess, int n,
 		}
 		break;
 
+	case 348:	/* +e-list entry */
+		if (!inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6], TRUE))
+			goto def;
+		break;
+
+	case 349:	/* end of exemption list */
+		sess = find_channel (serv, word[4]);
+		if (!sess)
+			sess = serv->front_session;
+		if (!fe_is_banwindow (sess))
+			goto def;
+		fe_ban_list_end (sess, TRUE);
+		break;
+
 	case 353:						  /* NAMES */
 		inbound_nameslist (serv, word[5],
 							(word_eol[6][0] == ':') ? word_eol[6] + 1 : word_eol[6]);
@@ -598,7 +639,7 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 367: /* banlist entry */
-		inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6]);
+		inbound_banlist (sess, atol (word[7]), word[4], word[5], word[6], FALSE);
 		break;
 
 	case 368:
@@ -607,7 +648,7 @@ process_numeric (session * sess, int n,
 			sess = serv->front_session;
 		if (!fe_is_banwindow (sess))
 			goto def;
-		fe_ban_list_end (sess);
+		fe_ban_list_end (sess, FALSE);
 		break;
 
 	case 372:	/* motd text */
@@ -664,6 +705,16 @@ process_numeric (session * sess, int n,
 		break;
 
 	default:
+
+		if (serv->inside_whois && word[4][0])
+		{
+			/* some unknown WHOIS reply, ircd coders make them up weekly */
+			EMIT_SIGNAL (XP_TE_WHOIS_SPECIAL, serv->server_session, word[4],
+							(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5],
+							 word[2], NULL, 0);
+			return;
+		}
+
 	def:
 		if (is_channel (serv, word[4]))
 		{
@@ -679,6 +730,127 @@ process_numeric (session * sess, int n,
 	}
 }
 
+static void
+dequote_ctcp (char *text)
+{
+#define M_QUOTE '\020'
+#define X_DELIM '\001'
+#define X_QUOTE '\134'
+	/* find and dequote low-level CTCP quoting */
+	char	*ptr = text;
+	int new_lines = 0;
+
+	if (prefs.text_max_newlines < 1)
+		return;
+
+	while ((ptr = strchr(ptr, M_QUOTE)) != NULL)
+	{
+		switch(*(ptr+1))
+		{
+		  case 'r':
+			*(ptr+1) = '\r';
+			break;
+		  case 'n':
+			if (*(ptr+2) != 0) /* don't allow trailing \n */
+			{
+				*(ptr+1) = '\n';
+				new_lines++;
+			} else
+			{
+				*(ptr+1) = 0;
+			}
+			break;
+		  case '0':
+			*(ptr+1) = 0;
+			break;
+		}
+		memmove(ptr, ptr+1, strlen(ptr+1)+1);
+		ptr++;
+
+		/* reached the limit? ignore the rest */
+		if (new_lines >= prefs.text_max_newlines)
+			break;
+	}
+}
+
+static void
+process_privmsg (session *sess, char *nick, char *ip, char *text, char *word[], char *word_eol[])
+{
+	server *serv = sess->server;
+	char *to = word[3];
+	int id = FALSE;	/* identified */
+
+	if (!(*to))
+		return;
+
+	text = word_eol[4];
+	if (*text == ':')
+		text++;
+	if (serv->have_idmsg)
+	{
+		if (*text == '+')
+		{
+			id = TRUE;
+			text++;
+		} else if (*text == '-')
+			text++;
+	}
+
+	dequote_ctcp (text);
+
+	{ /* find, handle, and remove CTCP messages */
+		char	*start, *end;
+
+		while (((start = strchr(text, X_DELIM)) != NULL) && ((end = strchr(start+1, X_DELIM)) != NULL))
+		{
+			*end = 0;
+
+			if (strncasecmp(start+1, "ACTION ", sizeof("ACTION ")-1) != 0)
+				flood_check(nick, ip, serv, sess, 0);
+			if (strncasecmp(text, "DCC ", sizeof("DCC ")-1) == 0)
+				/* redo this with handle_quotes TRUE */
+				process_data_init (word[1], word_eol[1], word, word_eol, TRUE);
+			else
+				/* redo this to update the word[] array (word_eol[] was changed via 'text' ptr) */
+				process_data_init (word[1], word_eol[1], word, word_eol, FALSE);
+
+			{ /* inline CTCP dequote */
+				char	*ptr = start+1;
+
+				while ((ptr = strchr(ptr, X_QUOTE)) != NULL)
+				{
+					switch(*(ptr+1))
+					{
+					  case 'a':
+						*(ptr+1) = X_DELIM;
+						break;
+					}
+					memmove(ptr, ptr+1, strlen(ptr+1)+1);
+					ptr++;
+				}
+			}
+
+			ctcp_handle(sess, to, nick, start+1, word, word_eol);
+			memmove(start, end+1, strlen(end+1)+1);
+		}
+	}
+
+	if (*text != 0)
+	{
+		if (is_channel (serv, to))
+		{
+			if (ignore_check (word[1], IG_CHAN))
+				return;
+			inbound_chanmsg (serv, NULL, to, nick, text, FALSE, id);
+		} else
+		{
+			if (ignore_check (word[1], IG_PRIV))
+				return;
+			inbound_privmsg (serv, nick, ip, text, id);
+		}
+	}
+}
+
 /* handle named messages that starts with a ':' */
 
 static void
@@ -773,49 +945,7 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 	}
 	if (!strcmp ("PRIVMSG", type))
 	{
-		char *to = word[3];
-		int len;
-		int id = FALSE;	/* identified */
-		if (*to)
-		{
-			text = word_eol[4];
-			if (*text == ':')
-				text++;
-			if (serv->have_idmsg)
-			{
-				if (*text == '+')
-				{
-					id = TRUE;
-					text++;
-				} else if (*text == '-')
-					text++;
-			}
-			len = strlen (text);
-			if (text[0] == 1 && text[len - 1] == 1)	/* ctcp */
-			{
-				text[len - 1] = 0;
-				text++;
-				if (strncasecmp (text, "ACTION", 6) != 0)
-					flood_check (nick, ip, serv, sess, 0);
-				if (strncasecmp (text, "DCC ", 4) == 0)
-					/* redo this with handle_quotes TRUE */
-					process_data_init (word[1], word_eol[1], word, word_eol, TRUE);
-				ctcp_handle (sess, to, nick, text, word, word_eol);
-			} else
-			{
-				if (is_channel (serv, to))
-				{
-					if (ignore_check (word[1], IG_CHAN))
-						return;
-					inbound_chanmsg (serv, NULL, to, nick, text, FALSE, id);
-				} else
-				{
-					if (ignore_check (word[1], IG_PRIV))
-						return;
-					inbound_privmsg (serv, nick, ip, text, id);
-				}
-			}
-		}
+		process_privmsg (sess, nick, ip, text, word, word_eol);
 		return;
 	}
 	if (!strcmp ("PONG", type))
@@ -981,6 +1111,7 @@ proto_fill_her_up (server *serv)
 	serv->p_quit = irc_quit;
 	serv->p_kick = irc_kick;
 	serv->p_part = irc_part;
+	serv->p_nickserv = irc_nickserv;
 	serv->p_join = irc_join;
 	serv->p_login = irc_login;
 	serv->p_join_info = irc_join_info;
