@@ -31,6 +31,7 @@
 #include "cfgfiles.h"
 #include "plugin.h"
 #include "fe.h"
+#include "server.h"
 #include "util.h"
 #include "outbound.h"
 #include "xchatc.h"
@@ -335,7 +336,7 @@ log_open (session *sess)
 
 	log_close (sess);
 	sess->logfd = log_open_file (sess->server->servername, sess->channel,
-										  get_network (sess, FALSE));
+										  server_get_network (sess->server, FALSE));
 
 	if (!log_error && sess->logfd == -1)
 	{
@@ -673,6 +674,7 @@ static char * const pevt_partreason_help[] = {
 static char * const pevt_ctcpsnd_help[] = {
 	N_("The sound"),
 	N_("The nick of the person"),
+	N_("The channel"),
 };
 
 static char * const pevt_ctcpgen_help[] = {
@@ -810,6 +812,7 @@ static char * const pevt_whois6_help[] = {
 static char * const pevt_whoisid_help[] = {
 	N_("Nickname"),
 	N_("Message"),
+	"Numeric"
 };
 
 static char * const pevt_whoisauth_help[] = {
@@ -1556,7 +1559,9 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	sound_play_event (index);
 
-	display_event (pntevts[index], sess, te[index].num_args, word);
+	/* If a plugin's callback executes "/close", 'sess' may be invalid */
+	if (is_session (sess))
+		display_event (pntevts[index], sess, te[index].num_args, word);
 }
 
 int
