@@ -24,7 +24,7 @@
 #include "../fe-gtk/xtext.h"
 #include "../common/xchat.h"
 #include "userlist.h"
-#include "navigation_tree.h"
+#include "navigation-tree.h"
 
 #include <gtk/gtkactiongroup.h>
 #include <gtk/gtkuimanager.h>
@@ -39,7 +39,6 @@ typedef struct
 	GnomeApp *main_window;
 	GtkActionGroup *action_group;
 	GtkUIManager *manager;
-	GtkDialog *preferences_dialog;
 	GnomeAbout *about;
 	GtkXText *xtext;
 #if (GTK_CHECK_VERSION(2,5,0))
