@@ -24,6 +24,7 @@
 #include "preferences.h"
 #include "../common/text.h"
 #include "../common/xchatc.h"
+#include "../common/fe.h"
 #include <libgnome/gnome-url.h> /* gnome_url_show */
 #include <gconf/gconf-client.h>
 
@@ -37,7 +38,7 @@ static void copy_text (GtkAction *action, gpointer data);
 static void send_email (GtkAction *action, gpointer data);
 
 static GHashTable *notify_table;
-static gchar *selected_word;
+static gchar *selected_word = NULL;
 
 static GtkActionEntry action_entries[] = {
 	/* URL Popup */
@@ -280,7 +281,9 @@ clicked_word (GtkWidget *xtext, char *word, GdkEventButton *event, gpointer data
 			return;
 		case WORD_URL:
 		case WORD_HOST:
-			selected_word = word;
+			if (selected_word)
+				g_free (selected_word);
+			selected_word = g_strdup (word);
 			open_url (NULL, NULL);
 			break;
 		}
@@ -301,7 +304,9 @@ clicked_word (GtkWidget *xtext, char *word, GdkEventButton *event, gpointer data
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextURLPopup");
 				g_return_if_fail (menu != NULL);
-				selected_word = word;
+				if (selected_word)
+					g_free (selected_word);
+				selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 				return;
 			}
@@ -316,7 +321,9 @@ clicked_word (GtkWidget *xtext, char *word, GdkEventButton *event, gpointer data
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextEmailPopup");
 				g_return_if_fail (menu != NULL);
-				selected_word = word;
+				if (selected_word)
+					g_free (selected_word);
+				selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 				return;
 			}
@@ -331,6 +338,7 @@ void
 font_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
 	gchar *font;
+	GtkAdjustment *adj;
 
 	if (gconf_client_get_bool (client, "/apps/xchat/main_window/use_sys_fonts", NULL))
 		font = gconf_client_get_string (client, "/desktop/gnome/interface/font_name", NULL);
@@ -338,6 +346,8 @@ font_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer us
 		font = gconf_client_get_string (client, "/apps/xchat/main_window/font", NULL);
 
 	gtk_xtext_set_font (GTK_XTEXT (gui.xtext), font);
+	adj = GTK_XTEXT (gui.xtext)->adj;
+	gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
 	gtk_xtext_refresh (GTK_XTEXT (gui.xtext), 0);
 
 	g_free (font);
